Exception: RightScale::Exceptions::ConnectivityFailure

Inherits:
NestedException show all
Defined in:
lib/right_agent/exceptions.rb

Overview

Cannot connect to service, lost connection to it, or it is out of service or too busy to respond

Instance Attribute Summary

Attributes inherited from NestedException

#nested_exception

Instance Method Summary collapse

Constructor Details

#initialize(message, nested_exception = nil) ⇒ ConnectivityFailure

Returns a new instance of ConnectivityFailure.



59
60
61
# File 'lib/right_agent/exceptions.rb', line 59

def initialize(message, nested_exception = nil)
  super(message, nested_exception)
end