Exception: RightScale::Exceptions::ConnectivityFailure
- Inherits:
-
NestedException
- Object
- StandardError
- NestedException
- RightScale::Exceptions::ConnectivityFailure
- Defined in:
- lib/right_agent/exceptions.rb
Overview
Cannot connect to service, lost connection to it, or it is out of service or too busy to respond
Instance Attribute Summary
Attributes inherited from NestedException
Instance Method Summary collapse
-
#initialize(message, nested_exception = nil) ⇒ ConnectivityFailure
constructor
A new instance of ConnectivityFailure.
Constructor Details
#initialize(message, nested_exception = nil) ⇒ ConnectivityFailure
Returns a new instance of ConnectivityFailure.
59 60 61 |
# File 'lib/right_agent/exceptions.rb', line 59 def initialize(, nested_exception = nil) super(, nested_exception) end |